/* -*- Mode: C; tab-width: 4; -*- */
/*******************************************************************************
 * npTango.c
 ******************************************************************************
 * Local Demon Plugin
 ******************************************************************************
 * OVERVIEW
 * --------
 * Section 0 - Includes
 * Section 1 - Utility Method Implementations
 * Section 2 - Instance Structs
 * Section 3 - API Plugin Implementations
 * Section 4 - Platform specific Implementations
 * Section 5 - Java Native Method Implementations
 *******************************************************************************/


/*******************************************************************************
 * SECTION 0 - Includes
 ******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "npapi.h"
/*------------------------------------------------------------------------------
 * Define IMPLEMENT_Simple before including Simple.h to state that we're
 * implementing the native methods of this plug-in here, and consequently
 * need to access it's protected and private memebers.
 *----------------------------------------------------------------------------*/
#define IMPLEMENT_webwisdom_tango_structures_LocalBase
#include "webwisdom_tango_structures_LocalBase.h"


#ifdef _WINDOWS /* Windows Includes */
#include <windows.h>
#endif /* _WINDOWS */


/*******************************************************************************
 * SECTION 1 - Utility method implementations
 ******************************************************************************/


#define MBOX_ERR "Tango Error"
#define MBOX_DBG "dbgFun"

#define REG_ROOT_DIR "SOFTWARE\\WebWisdom\\Path1.0"

BOOL getRegistryKey(char *regKeyName,LPBYTE regVal,DWORD regValLen)
{
	HKEY regKeyHandle;
	int s;


	s=RegOpenKeyEx(HKEY_LOCAL_MACHINE,regKeyName,0,KEY_QUERY_VALUE,&regKeyHandle);
	if(s!=ERROR_SUCCESS)
	{
		fprintf(stderr,"getApplicationPath(): RegOpenKeyEx(%s) failed (%d)!",regKeyName,s);
		//MessageBox( NULL, "Could not read a value of WebWisdom local application directory", "Opening a key in registry failed", MB_ICONEXCLAMATION );
		return FALSE;
	}

	s=RegQueryValueEx(regKeyHandle,"",NULL,NULL,regVal,&regValLen);
	if(s!=ERROR_SUCCESS)
	{
		fprintf(stderr,"getApplicationPath(): RegQueryValueKey() failed (%d)!",s);
		//MessageBox( NULL, "Could not read a value of WebWisdom local application directory", "Querying value in registry failed", MB_ICONEXCLAMATION );
		return FALSE;
	}

	s=RegCloseKey(regKeyHandle);
	if(s!=ERROR_SUCCESS)	
	{
		fprintf(stderr,"getApplicationPath(): RegCloseKey() failed (%d)!",s);
		//MessageBox( NULL, "Could not read a value of WebWisdom local application directory", "Closing a key in registry failed", MB_ICONEXCLAMATION );
		return TRUE;
	}
	
	return TRUE;
}// getApplicationPath __________________________________________________


/*******************************************************************************
 * SECTION 2 - Instance Structs
 *******************************************************************************
 * Instance state information about the plugin.
 *
 * PLUGIN DEVELOPERS:
 *	Use this struct to hold per-instance information that you'll
 *	need in the various functions in this file.
 ******************************************************************************
 * First comes the PlatformInstance struct, which contains platform specific
 * information for each instance.
 *****************************************************************************/
 
/*------------------------------------------------------------------------------
 * Windows PlatformInstance
 *----------------------------------------------------------------------------*/
#ifdef XP_PC
#ifdef TRASH_
typedef struct _PlatformInstance
{
	HWND			fhWnd;
	WNDPROC			fDefaultWindowProc;
} PlatformInstance;
#endif /*TRASH_*/
#endif /* XP_PC */

#ifdef TRASH_
/*------------------------------------------------------------------------------
 * Cross-Platform PluginInstance
 *----------------------------------------------------------------------------*/
typedef struct _PluginInstance
{
	NPWindow*			fWindow;
	uint16				fMode;
	char*				fOutString;
	PlatformInstance	fPlatform;
} PluginInstance;
#endif /*TRASH_*/

char* serverHost;
char* serverPort;
char* localPort;
char* cmdExe;


/******************************************************************************
 * Method Declarations
 ******************************************************************************/
#ifdef TRASH_
void		PlatformNew( PluginInstance* This );
NPError		PlatformDestroy( PluginInstance* This );
NPError		PlatformSetWindow( PluginInstance* This, NPWindow* window );
int16		PlatformHandleEvent( PluginInstance* This, void* event );
#endif /*TRASH_*/

/*******************************************************************************
 * SECTION 3 - API Plugin Implementations
 ******************************************************************************/


/*------------------------------------------------------------------------------
 * Cross-Platform Plug-in API Calls
 *----------------------------------------------------------------------------*/
 
/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * NPP_Initialize:
 * Provides global initialization for a plug-in, and returns an error value. 
 *
 * This function is called once when a plug-in is loaded, before the first instance
 * is created. You should allocate any memory or resources shared by all
 * instances of your plug-in at this time. After the last instance has been deleted,
 * NPP_Shutdown will be called, where you can release any memory or
 * resources allocated by NPP_Initialize. 
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
NPError NPP_Initialize(void)
{
    return NPERR_NO_ERROR;
}// NPP_Initialize _______________________________________________


/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * NPP_GetJavaClass:
 * New in Netscape Navigator 3.0. 
 *
 * NPP_GetJavaClass is called during initialization to ask your plugin
 * what its associated Java class is. If you don't have one, just return
 * NULL. Otherwise, use the javah-generated "use_" function to both
 * initialize your class and return it. If you can't find your class, an
 * error will be signalled by "use_" and will cause the Navigator to
 * complain to the user.
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
jref NPP_GetJavaClass(void)
{
	struct java_lang_Class* myClass;
	JRIEnv* env;

	//MessageBox(NULL,"NPP_GetJavaClass",MBOX_DBG,MB_ICONEXCLAMATION);

	env=NPN_GetJavaEnv();
	if(env==NULL)
	{
		MessageBox(NULL,"NPP_GetJavaClass: Java disabled!",MBOX_ERR,MB_ICONEXCLAMATION);
		return NULL; /* Java disabled */
	}

    myClass=use_webwisdom_tango_structures_LocalBase(env);
	if(myClass==NULL)
	{
		MessageBox(NULL,"NPP_GetJavaClass: LocalBase not installed!",MBOX_ERR,MB_ICONEXCLAMATION);
		return NULL;
	}

	return (jref)myClass;
}// NPP_GetJavaClass _____________________________________________


/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * NPP_Shutdown:
 * Provides global deinitialization for a plug-in. 
 * 
 * This function is called once after the last instance of your plug-in is destroyed.
 * Use this function to release any memory or resources shared across all
 * instances of your plug-in. You should be a good citizen and declare that
 * you're not using your java class any more. This allows java to unload
 * it, freeing up memory.
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
void NPP_Shutdown(void)
{
	JRIEnv* env;
	env=NPN_GetJavaEnv();
	if(env)
	{
		unuse_webwisdom_tango_structures_LocalBase(env);
	}
}// NPP_Shutdown ___________________________________________________


/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * NPP_New:
 * Creates a new instance of a plug-in and returns an error value. 
 * 
 * NPP_New creates a new instance of your plug-in with MIME type specified
 * by pluginType. The parameter mode is NP_EMBED if the instance was created
 * by an EMBED tag, or NP_FULL if the instance was created by a separate file.
 * You can allocate any instance-specific private data in instance->pdata at this
 * time. The NPP pointer is valid until the instance is destroyed. 
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
NPError NPP_New(NPMIMEType pluginType,
	NPP instance,
	uint16 mode,
	int16 argc,
	char* argn[],
	char* argv[],
	NPSavedData* saved)
{
	NPError result = NPERR_NO_ERROR;
	//PluginInstance* This;

	//MessageBox(NULL,"NPP_New",MBOX_DBG,MB_ICONEXCLAMATION);

	if(instance==NULL)
		return NPERR_INVALID_INSTANCE_ERROR;
	//instance->pdata = NPN_MemAlloc(sizeof(PluginInstance));
	//This = (PluginInstance*) instance->pdata;
	//if (This == NULL) {
	//    return NPERR_OUT_OF_MEMORY_ERROR;
	//}
	/* mode is NP_EMBED, NP_FULL, or NP_BACKGROUND (see npapi.h) */
	//This->fWindow = NULL;
	//This->fMode = mode;

	
	//PlatformNew( This ); 	/* Call Platform-specific initializations */

		/* PLUGIN DEVELOPERS:
		 *	Initialize fields of your plugin
		 *	instance data here.  If the NPSavedData is non-
		 *	NULL, you can use that data (returned by you from
		 *	NPP_Destroy to set up the new plugin instance).
		 */
	serverHost=NULL;
	serverPort=NULL;
	localPort=NULL;
	{
		int i;
		//char c[256];

		//sprintf(c,"argc=%d",argc);
		//MessageBox(NULL,c,"appletParam#",MB_ICONEXCLAMATION);
		
		for(i=0;i<argc;i++)
		{
			//sprintf(c,"[%d]\"%s\"=\"%s\"",i,argn[i],argv[i]);
			//MessageBox(NULL,c,"appletParam",MB_ICONEXCLAMATION);

			if(strcmp(_strupr(argn[i]),"SERVERHOST")==0)
			{
				serverHost=NPN_MemAlloc(strlen(argv[i])+1);
				if(serverHost==NULL)
					return NPERR_OUT_OF_MEMORY_ERROR;
				strcpy(serverHost,argv[i]);
			}
			else if(strcmp(_strupr(argn[i]),"SERVERPORT")==0)
			{
				serverPort=NPN_MemAlloc(strlen(argv[i])+1);
				if(serverPort==NULL)
					return NPERR_OUT_OF_MEMORY_ERROR;
				strcpy(serverPort,argv[i]);
			}
			else if(strcmp(_strupr(argn[i]),"LOCALPORT")==0)
			{
				localPort=NPN_MemAlloc(strlen(argv[i])+1);
				if(localPort==NULL)
					return NPERR_OUT_OF_MEMORY_ERROR;
				strcpy(localPort,argv[i]);
			}
		}
	}

	cmdExe=NULL;
	{
		/*DWORD GetEnvironmentVariable(
			LPCTSTR lpName,// address of environment variable name 
			LPTSTR lpBuffer,// address of buffer for variable value
			DWORD nSize // size of buffer, in characters 
			);*/
		const char *cmdExeEnv="COMSPEC";
		DWORD bufLen=128,strLen;
		int i;
		for(i=0;i<4;i++)
		{
			cmdExe=NPN_MemAlloc(bufLen);
			if(cmdExe==NULL)
				return NPERR_OUT_OF_MEMORY_ERROR;
		    strLen=GetEnvironmentVariable(cmdExeEnv,cmdExe,bufLen);
			if(strLen==bufLen)
			{
				NPN_MemFree(cmdExe);
				cmdExe=NULL;
				bufLen*=2;
				continue;
			}
			else
			{
				break;
			}
		}
		if(strLen==0)
		{
			char c[128];
			sprintf(c,"%s: no such variable",cmdExeEnv);
			MessageBox(NULL,c,MBOX_ERR,MB_OK);
			NPN_MemFree(cmdExe);
			cmdExe=NULL;
			return NPERR_NO_ERROR;
		}
		else if(strLen==bufLen)
		{
			char c[128];
			sprintf(c,"unexpectedly long path");
			MessageBox(NULL,c,MBOX_ERR,MB_OK);
			NPN_MemFree(cmdExe);
			cmdExe=NULL;
			return NPERR_NO_ERROR;
		}
		/*{
			char c[256];
			sprintf(c,"cmdExe=%s",cmdExe);
			MessageBox(NULL,c,MBOX_DBG,MB_OK);
		}*/
	}
	return result;
}// NPP_New ________________________________________________________


/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * NPP_Destroy:
 * Deletes a specific instance of a plug-in and returns an error value. 

 * NPP_Destroy is called when a plug-in instance is deleted, typically because the
 * user has left the page containing the instance, closed the window, or quit the
 * application. You should delete any private instance-specific information stored
 * in instance->pdata. If the instance being deleted is the last instance created
 * by your plug-in, NPP_Shutdown will subsequently be called, where you can
 * delete any data allocated in NPP_Initialize to be shared by all your plug-in's
 * instances. Note that you should not perform any graphics operations in
 * NPP_Destroy as the instance's window is no longer guaranteed to be valid. 
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
NPError 
NPP_Destroy(NPP instance, NPSavedData** save)
{
	//MessageBox(NULL,"NPP_Destroy",MBOX_DBG,MB_ICONEXCLAMATION);
	
	if(instance==NULL)
		return NPERR_INVALID_INSTANCE_ERROR;

	/* PLUGIN DEVELOPERS:
	 *	If desired, call NP_MemAlloc to create a
	 *	NPSavedDate structure containing any state information
	 *	that you want restored if this plugin instance is later
	 *	recreated.
	 */

	if(serverHost!=NULL)
	{
		NPN_MemFree(serverHost);
		serverHost=NULL;
	} 
	if(serverPort!=NULL)
	{
		NPN_MemFree(serverPort);
		serverPort=NULL;
	} 
	if(localPort!=NULL)
	{
		NPN_MemFree(localPort);
		localPort=NULL;
	}
	if(cmdExe!=NULL)
	{
		NPN_MemFree(cmdExe);
		cmdExe=NULL;
	}

	return NPERR_NO_ERROR;
}// NPP_Destroy ________________________________________________


/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * NPP_SetWindow:
 * Sets the window in which a plug-in draws, and returns an error value. 
 * 
 * NPP_SetWindow informs the plug-in instance specified by instance of the
 * the window denoted by window in which the instance draws. This NPWindow
 * pointer is valid for the life of the instance, or until NPP_SetWindow is called
 * again with a different value. Subsequent calls to NPP_SetWindow for a given
 * instance typically indicate that the window has been resized. If either window
 * or window->window are NULL, the plug-in must not perform any additional
 * graphics operations on the window and should free any resources associated
 * with the window. 
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
NPError 
NPP_SetWindow(NPP instance,NPWindow* window)
{
	if(instance==NULL)
		return NPERR_INVALID_INSTANCE_ERROR;
	/*
	 * PLUGIN DEVELOPERS:
	 *	Before setting window to point to the
	 *	new window, you may wish to compare the new window
	 *	info to the previous window (if any) to note window
	 *	size changes, etc.
	 */
	return NPERR_NO_ERROR;
}// NPP_setWindow ________________________________________________


/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * NPP_NewStream:
 * Notifies an instance of a new data stream and returns an error value. 
 * 
 * NPP_NewStream notifies the instance denoted by instance of the creation of
 * a new stream specifed by stream. The NPStream* pointer is valid until the
 * stream is destroyed. The MIME type of the stream is provided by the
 * parameter type. 
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
NPError 
NPP_NewStream(NPP instance,
	      NPMIMEType type,
	      NPStream *stream, 
	      NPBool seekable,
	      uint16 *stype)
{
	if(instance==NULL)
		return NPERR_INVALID_INSTANCE_ERROR;
	return NPERR_NO_ERROR;
}// NPP_NewStream ___________________________________________________


/* PLUGIN DEVELOPERS:
 *	These next 2 functions are directly relevant in a plug-in which
 *	handles the data in a streaming manner. If you want zero bytes
 *	because no buffer space is YET available, return 0. As long as
 *	the stream has not been written to the plugin, Navigator will
 *	continue trying to send bytes.  If the plugin doesn't want them,
 *	just return some large number from NPP_WriteReady(), and
 *	ignore them in NPP_Write().  For a NP_ASFILE stream, they are
 *	still called but can safely be ignored using this strategy.
 */


/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * NPP_WriteReady:
 * Returns the maximum number of bytes that an instance is prepared to accept
 * from the stream. 
 * 
 * NPP_WriteReady determines the maximum number of bytes that the
 * instance will consume from the stream in a subsequent call NPP_Write. This
 * function allows Netscape to only send as much data to the instance as the
 * instance is capable of handling at a time, allowing more efficient use of
 * resources within both Netscape and the plug-in. 
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
int32 
NPP_WriteReady(NPP instance, NPStream *stream)
{
	return 0X0FFFFFFF;/* Number of bytes ready to accept in NPP_Write() */
}// NPP_WriteReady ___________________________________________________


/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * NPP_Write:
 * Delivers data from a stream and returns the number of bytes written. 
 * 
 * NPP_Write is called after a call to NPP_NewStream in which the plug-in
 * requested a normal-mode stream, in which the data in the stream is delivered
 * progressively over a series of calls to NPP_WriteReady and NPP_Write. The
 * function delivers a buffer buf of len bytes of data from the stream identified
 * by stream to the instance. The parameter offset is the logical position of
 * buf from the beginning of the data in the stream. 
 * 
 * The function returns the number of bytes written (consumed by the instance).
 * A negative return value causes an error on the stream, which will
 * subsequently be destroyed via a call to NPP_DestroyStream. 
 * 
 * Note that a plug-in must consume at least as many bytes as it indicated in the
 * preceeding NPP_WriteReady call. All data consumed must be either processed
 * immediately or copied to memory allocated by the plug-in: the buf parameter
 * is not persistent. 
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
int32 
NPP_Write(NPP instance, NPStream *stream, int32 offset, int32 len, void *buffer)
{
	return len;		/* The number of bytes accepted */
}// NPP_Write __________________________________________________________


/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * NPP_DestroyStream:
 * Indicates the closure and deletion of a stream, and returns an error value. 
 * 
 * The NPP_DestroyStream function is called when the stream identified by
 * stream for the plug-in instance denoted by instance will be destroyed. You
 * should delete any private data allocated in stream->pdata at this time. 
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
NPError 
NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
{
	if(instance==NULL)
		return NPERR_INVALID_INSTANCE_ERROR;
	return NPERR_NO_ERROR;
}// NPP_DestroyStream ____________________________________________________


/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * NPP_StreamAsFile:
 * Provides a local file name for the data from a stream. 
 * 
 * NPP_StreamAsFile provides the instance with a full path to a local file,
 * identified by fname, for the stream specified by stream. NPP_StreamAsFile is
 * called as a result of the plug-in requesting mode NP_ASFILEONLY or
 * NP_ASFILE in a previous call to NPP_NewStream. If an error occurs while
 * retrieving the data or writing the file, fname may be NULL. 
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
void 
NPP_StreamAsFile(NPP instance, NPStream *stream, const char* fname)
{
	return;
}// NPP_StreamAsFile _______________________________________________________


/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * NPP_Print:
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
void 
NPP_Print(NPP instance, NPPrint* printInfo)
{
	return;
}// NPP_Print _______________________________________________________________


/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * NPP_URLNotify:
 * Notifies the instance of the completion of a URL request. 
 * 
 * NPP_URLNotify is called when Netscape completes a NPN_GetURLNotify or
 * NPN_PostURLNotify request, to inform the plug-in that the request,
 * identified by url, has completed for the reason specified by reason. The most
 * common reason code is NPRES_DONE, indicating simply that the request
 * completed normally. Other possible reason codes are NPRES_USER_BREAK,
 * indicating that the request was halted due to a user action (for example,
 * clicking the "Stop" button), and NPRES_NETWORK_ERR, indicating that the
 * request could not be completed (for example, because the URL could not be
 * found). The complete list of reason codes is found in npapi.h. 
 * 
 * The parameter notifyData is the same plug-in-private value passed as an
 * argument to the corresponding NPN_GetURLNotify or NPN_PostURLNotify
 * call, and can be used by your plug-in to uniquely identify the request. 
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
void
NPP_URLNotify(NPP instance, const char* url, NPReason reason, void* notifyData)
{
	return;
}// NPP_URLNotify ___________________________________________________________


/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * NPP_HandleEvent:
 * Mac-only, but stub must be present for Windows
 * Delivers a platform-specific event to the instance. 
 * 
 * On the Macintosh, event is a pointer to a standard Macintosh EventRecord.
 * All standard event types are passed to the instance as appropriate. In general,
 * return TRUE if you handle the event and FALSE if you ignore the event. 
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
#ifndef XP_UNIX
int16
NPP_HandleEvent(NPP instance, void* event)
{
	return 0;
}
#endif /* ndef XP_UNIX */
/*******************************************************************************/





/*******************************************************************************
 * SECTION 5 - Platform-Specific Implemenation
 ******************************************************************************/

/*------------------------------------------------------------------------------
 * 
 *----------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------
 * Windows Implementations
 *----------------------------------------------------------------------------*/
#ifdef XP_PC
#ifdef TRASH_
LRESULT CALLBACK PluginWindowProc( HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam);
const char* gInstanceLookupString = "instance->pdata";

/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * PlatformNew
 *
 * Initialize any Platform-Specific instance data.
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
void
PlatformNew( PluginInstance* This )
{
	This->fPlatform.fhWnd = NULL;
	This->fPlatform.fDefaultWindowProc = NULL;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * PlatformDestroy
 *
 * Destroy any Platform-Specific instance data.
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
NPError
PlatformDestroy( PluginInstance* This )
{
	if( This->fWindow != NULL ) { //If we have a window, clean it up.
		SetWindowLong( This->fPlatform.fhWnd, GWL_WNDPROC, (LONG)This->fPlatform.fDefaultWindowProc);
		This->fPlatform.fDefaultWindowProc = NULL;
		This->fPlatform.fhWnd = NULL;
	}
	return NPERR_NO_ERROR;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * PlatformSetWindow
 *
 * Perform platform-specific window operations
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
NPError
PlatformSetWindow( PluginInstance* This, NPWindow* window )
{
	if( This->fWindow != NULL ) /* If we already have a window, clean
								 * it up before trying to subclass
								 * the new window. */
	{
		if( (window == NULL) || ( window->window == NULL ) ) {
			/* There is now no window to use. get rid of the old
			 * one and exit. */
			SetWindowLong( This->fPlatform.fhWnd, GWL_WNDPROC, (LONG)This->fPlatform.fDefaultWindowProc);
			This->fPlatform.fDefaultWindowProc = NULL;
			This->fPlatform.fhWnd = NULL;
			return NPERR_NO_ERROR;
		}

		else if ( This->fPlatform.fhWnd == (HWND) window->window ) {
			/* The new window is the same as the old one. Redraw and get out. */

			InvalidateRect( This->fPlatform.fhWnd, NULL, TRUE );

			UpdateWindow( This->fPlatform.fhWnd );


			return NPERR_NO_ERROR;
		}
		else {
			/* Clean up the old window, so that we can subclass the new
			 * one later. */
			SetWindowLong( This->fPlatform.fhWnd, GWL_WNDPROC, (LONG)This->fPlatform.fDefaultWindowProc);
			This->fPlatform.fDefaultWindowProc = NULL;
			This->fPlatform.fhWnd = NULL;
		}
	}
	else if( (window == NULL) || ( window->window == NULL ) ) {
		/* We can just get out of here if there is no current
		 * window and there is no new window to use. */
		return NPERR_NO_ERROR;
	}

	/* At this point, we will subclass
	 * window->window so that we can begin drawing and
	 * receiving window messages. */
	This->fPlatform.fDefaultWindowProc = (WNDPROC)SetWindowLong( (HWND)window->window, GWL_WNDPROC, (LONG)PluginWindowProc);
	This->fPlatform.fhWnd = (HWND) window->window;
	SetProp( This->fPlatform.fhWnd, gInstanceLookupString, (HANDLE)This);

	InvalidateRect( This->fPlatform.fhWnd, NULL, TRUE );
	UpdateWindow( This->fPlatform.fhWnd );
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * PlatformHandleEvent
 *
 * Handle platform-specific events.
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
int16
PlatformHandleEvent( PluginInstance* This, void* event )
{
	/* Windows Plugins use the Windows event call-back mechanism
	   for events. (See PluginWindowProc) */
	return 0;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * PluginWindowProc
 *
 * Handle the Windows window-event loop.
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
LRESULT CALLBACK PluginWindowProc( HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	PluginInstance* This = (PluginInstance*) GetProp(hWnd, gInstanceLookupString);

	switch( Msg ) {
		case WM_PAINT: {
			PAINTSTRUCT paintStruct;
			HDC hdc;

			hdc = BeginPaint( hWnd, &paintStruct );
			TextOut(hdc, 0, 0, "", 0);

			EndPaint( hWnd, &paintStruct );
			break;
		}
		default: {
			This->fPlatform.fDefaultWindowProc( hWnd, Msg, wParam, lParam);
		}
	}
	return 0;
}


#endif /*TRASH_*/
#endif /* XP_PC */



/*******************************************************************************
 * SECTION 5 - Java Native Method Implementations
 ******************************************************************************/


/******************************************************************************/
/*
void SetStart(LPSTARTUPINFO p)
{
	p->cb = sizeof(STARTUPINFO); 
    p->lpReserved = NULL; 
    p->lpDesktop = NULL; 
    p->lpTitle = NULL; 
    p->dwFlags = 0;     
    p->cbReserved2 = 0; 
    p->lpReserved2 =NULL; 
}*/


JRI_PUBLIC_API(jint)								 				
native_webwisdom_tango_structures_LocalBase_runApplication(JRIEnv* env,struct webwisdom_tango_structures_LocalBase* self,struct java_lang_String *a, jint AID, jint SID, jint AT, jint PORT)
{
	char rootDir[_MAX_PATH];
	const char *progFile=JRI_GetStringUTFChars(env,a);
	char cmdLine[_MAX_PATH];
	char cmdPath[_MAX_PATH];
	char cmdParams[128];
	
	//MessageBox(NULL,"runApplication",MBOX_DBG,MB_ICONEXCLAMATION);

	if(progFile==NULL)
	{
		return -2;
	}

	if(getRegistryKey(REG_ROOT_DIR,rootDir,_MAX_PATH)!=TRUE)
	{
		char c[128];
		sprintf(c,"Cannot read registry key:\n%s",REG_ROOT_DIR);
		MessageBox(NULL,c,MBOX_ERR,MB_OK);
		return -1;
	}
	sprintf(cmdPath, "\"%s%s\"", rootDir, progFile);
	sprintf(cmdParams," %d %d %d %d", AID,SID,AT,PORT);
	sprintf(cmdLine, "%s%s", cmdPath, cmdParams);
	{
		/*BOOL CreateProcess( 
			LPCTSTR lpApplicationName, // pointer to name of executable module 
			LPTSTR lpCommandLine, // pointer to command line string 
			LPSECURITY_ATTRIBUTES lpProcessAttributes, // pointer to process security attributes 
			LPSECURITY_ATTRIBUTES lpThreadAttributes,// pointer to thread security attributes 
			BOOL bInheritHandles, // handle inheritance flag 
			DWORD dwCreationFlags, // creation flags 
			LPVOID lpEnvironment, // pointer to new environment block 
			LPCTSTR lpCurrentDirectory,// pointer to current directory name 
			LPSTARTUPINFO lpStartupInfo, // pointer to STARTUPINFO 
			LPPROCESS_INFORMATION lpProcessInformation // pointer to PROCESS_INFORMATION 
			);*/
		PROCESS_INFORMATION procInfo;
		STARTUPINFO startInfo;
		BOOL status;

		memset(&startInfo,0,sizeof(STARTUPINFO));
		startInfo.cb=sizeof(STARTUPINFO);
		startInfo.wShowWindow=SW_SHOW;

		startInfo.cb=sizeof(STARTUPINFO); 
		startInfo.lpReserved=NULL; 
		startInfo.lpDesktop=NULL; 
		startInfo.lpTitle=NULL; 
		startInfo.dwFlags=0;     
		startInfo.cbReserved2=0; 
		startInfo.lpReserved2=NULL; 
	
		status=CreateProcess(NULL,cmdLine,NULL,NULL,FALSE,
		/*0*/NORMAL_PRIORITY_CLASS|DETACHED_PROCESS,NULL,NULL/*rootDir*/,&startInfo,&procInfo); 

		if(status!=TRUE)
		{
			WIN32_FIND_DATA findFileData;
			char batFile[_MAX_PATH];
			sprintf(batFile, "%s%s.bat", rootDir, progFile);
			/*look for a batch file*/
			/*if that also fails, scream*/
			if(FindFirstFile(batFile, &findFileData)==INVALID_HANDLE_VALUE)
			{
				char buf[256];
				sprintf(buf, 
					"Could not start the local application.\nPartial path was: %s",
					progFile);
				MessageBox(NULL, buf, MBOX_ERR, MB_ICONSTOP);
				return -1;
			}
			else
			{
				sprintf(cmdLine, "\"%s\"%s", batFile, cmdParams);
				system(cmdLine);
			}
		}
	}

	return 0;
}// runApplication ______________________________________________________


JRI_PUBLIC_API(struct java_lang_String *)   
native_webwisdom_tango_structures_LocalBase_getServerHost(JRIEnv* env,struct webwisdom_tango_structures_LocalBase* self)
{
    struct java_lang_String *jHost;
 	//MessageBox(NULL,"getServerHost",MBOX_DBG,MB_ICONEXCLAMATION);
    if(serverHost==NULL)
	{	
		char host[2];
		host[0]='*';
		host[1]='\0';
	    jHost=(struct java_lang_String *)JRI_NewStringUTF(env,host,(jsize)strlen(host)); 
	}
	else
	{
	    jHost=(struct java_lang_String *)JRI_NewStringUTF(env,serverHost,(jsize)strlen(serverHost)); 
	}
    return jHost;
}// getServerHost ______________________________________________________


JRI_PUBLIC_API(jint)  
native_webwisdom_tango_structures_LocalBase_getServerPort(JRIEnv* env, webwisdom_tango_structures_LocalBase* self)
{
	//MessageBox(NULL,"getServerPort",MBOX_DBG,MB_ICONEXCLAMATION);
	if(serverPort==NULL)
		return -1;
	return atoi(serverPort);
}// getServerPort _____________________________________________________


JRI_PUBLIC_API(jint)   
native_webwisdom_tango_structures_LocalBase_getLocalPort(JRIEnv* env, webwisdom_tango_structures_LocalBase* self)
{
	//MessageBox(NULL,"getLocalPort",MBOX_DBG,MB_ICONEXCLAMATION);
    if(localPort==NULL)
		return -1;
    return atoi(localPort); 
}// getLocalPort ______________________________________________________


